@@ -72,6 +72,8 @@ typedef enum {
     S2N_ERR_DECRYPT,
     S2N_ERR_BAD_MESSAGE,
     S2N_ERR_UNEXPECTED_CERT_REQUEST,
+    S2N_ERR_MISSING_CERT_REQUEST,
+    S2N_ERR_MISSING_CLIENT_CERT,
     S2N_ERR_KEY_INIT,
     S2N_ERR_KEY_DESTROY,
     S2N_ERR_DH_SERIALIZING,
