@@ -102,7 +102,7 @@ struct s2n_connection *s2n_connection_new(s2n_mode mode)
     PTR_GUARD_POSIX(s2n_blob_init(&blob, conn->header_in_data, S2N_TLS_RECORD_HEADER_LENGTH));
     PTR_GUARD_POSIX(s2n_stuffer_init(&conn->header_in, &blob));
     PTR_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->out, 0));
-    PTR_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->in, 0));
+    PTR_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->buffer_in, 0));
     PTR_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->handshake.io, 0));
     PTR_GUARD_RESULT(s2n_timer_start(conn->config, &conn->write_timer));
 
@@ -262,6 +262,7 @@ int s2n_connection_free(struct s2n_connection *conn)
     POSIX_GUARD(s2n_free(&conn->peer_quic_transport_parameters));
     POSIX_GUARD(s2n_free(&conn->server_early_data_context));
     POSIX_GUARD(s2n_free(&conn->tls13_ticket_fields.session_secret));
+    POSIX_GUARD(s2n_stuffer_free(&conn->buffer_in));
     POSIX_GUARD(s2n_stuffer_free(&conn->in));
     POSIX_GUARD(s2n_stuffer_free(&conn->out));
     POSIX_GUARD(s2n_stuffer_free(&conn->handshake.io));
@@ -407,7 +408,9 @@ int s2n_connection_release_buffers(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_resize(&conn->out, 0));
 
     POSIX_ENSURE(s2n_stuffer_is_consumed(&conn->in), S2N_ERR_STUFFER_HAS_UNPROCESSED_DATA);
-    POSIX_GUARD(s2n_stuffer_resize(&conn->in, 0));
+    if (s2n_stuffer_is_consumed(&conn->buffer_in)) {
+        POSIX_GUARD(s2n_stuffer_resize(&conn->buffer_in, 0));
+    }
 
     POSIX_ENSURE(s2n_stuffer_is_consumed(&conn->post_handshake.in), S2N_ERR_STUFFER_HAS_UNPROCESSED_DATA);
     POSIX_GUARD(s2n_stuffer_free(&conn->post_handshake.in));
@@ -464,7 +467,7 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     struct s2n_stuffer client_ticket_to_decrypt = { 0 };
     struct s2n_stuffer handshake_io = { 0 };
     struct s2n_stuffer header_in = { 0 };
-    struct s2n_stuffer in = { 0 };
+    struct s2n_stuffer buffer_in = { 0 };
     struct s2n_stuffer out = { 0 };
 
     /* Some required structures might have been freed to conserve memory between handshakes.
@@ -501,11 +504,12 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_wipe(&conn->post_handshake.in));
     POSIX_GUARD(s2n_blob_zero(&conn->client_hello.raw_message));
     POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
-    POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
+    POSIX_GUARD(s2n_stuffer_wipe(&conn->buffer_in));
     POSIX_GUARD(s2n_stuffer_wipe(&conn->out));
 
     /* Free stuffers we plan to just recreate */
     POSIX_GUARD(s2n_stuffer_free(&conn->post_handshake.in));
+    POSIX_GUARD(s2n_stuffer_free(&conn->in));
 
     POSIX_GUARD_RESULT(s2n_psk_parameters_wipe(&conn->psk_params));
 
@@ -526,7 +530,7 @@ int s2n_connection_wipe(struct s2n_connection *conn)
 
     /* Truncate the message buffers to save memory, we will dynamically resize it as needed */
     POSIX_GUARD(s2n_free(&conn->client_hello.raw_message));
-    POSIX_GUARD(s2n_stuffer_resize(&conn->in, 0));
+    POSIX_GUARD(s2n_stuffer_resize(&conn->buffer_in, 0));
     POSIX_GUARD(s2n_stuffer_resize(&conn->out, 0));
 
     /* Remove context associated with connection */
@@ -545,7 +549,7 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     POSIX_CHECKED_MEMCPY(&client_ticket_to_decrypt, &conn->client_ticket_to_decrypt, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&handshake_io, &conn->handshake.io, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&header_in, &conn->header_in, sizeof(struct s2n_stuffer));
-    POSIX_CHECKED_MEMCPY(&in, &conn->in, sizeof(struct s2n_stuffer));
+    POSIX_CHECKED_MEMCPY(&buffer_in, &conn->buffer_in, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&out, &conn->out, sizeof(struct s2n_stuffer));
 #ifdef S2N_DIAGNOSTICS_POP_SUPPORTED
     #pragma GCC diagnostic pop
@@ -557,9 +561,14 @@ int s2n_connection_wipe(struct s2n_connection *conn)
     POSIX_CHECKED_MEMCPY(&conn->client_ticket_to_decrypt, &client_ticket_to_decrypt, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&conn->handshake.io, &handshake_io, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&conn->header_in, &header_in, sizeof(struct s2n_stuffer));
-    POSIX_CHECKED_MEMCPY(&conn->in, &in, sizeof(struct s2n_stuffer));
+    POSIX_CHECKED_MEMCPY(&conn->buffer_in, &buffer_in, sizeof(struct s2n_stuffer));
     POSIX_CHECKED_MEMCPY(&conn->out, &out, sizeof(struct s2n_stuffer));
 
+    /* conn->in will eventually point to part of conn->buffer_in, but we initialize
+     * it as growable and allocated to support legacy tests.
+     */
+    POSIX_GUARD(s2n_stuffer_growable_alloc(&conn->in, 0));
+
     conn->handshake.hashes = handshake_hashes;
     conn->prf_space = prf_workspace;
     conn->initial = initial;
@@ -1628,13 +1637,13 @@ S2N_RESULT s2n_connection_dynamic_free_in_buffer(struct s2n_connection *conn)
 {
     RESULT_ENSURE_REF(conn);
 
-    /* free the `in` buffer if we're in dynamic mode and it's completely flushed */
-    if (conn->dynamic_buffers && s2n_stuffer_is_consumed(&conn->in)) {
+    /* free `buffer_in` if we're in dynamic mode and it's completely flushed */
+    if (conn->dynamic_buffers && s2n_stuffer_is_consumed(&conn->buffer_in)) {
         /* when copying the buffer into the application, we use `s2n_stuffer_erase_and_read`, which already zeroes the memory */
-        RESULT_GUARD_POSIX(s2n_stuffer_free_without_wipe(&conn->in));
+        RESULT_GUARD_POSIX(s2n_stuffer_free_without_wipe(&conn->buffer_in));
 
         /* reset the stuffer to its initial state */
-        RESULT_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->in, 0));
+        RESULT_GUARD_POSIX(s2n_stuffer_growable_alloc(&conn->buffer_in, 0));
     }
 
     return S2N_RESULT_OK;
@@ -1726,3 +1735,12 @@ int s2n_connection_get_key_update_counts(struct s2n_connection *conn,
     *recv_key_updates = conn->recv_key_updated;
     return S2N_SUCCESS;
 }
+
+int s2n_connection_set_recv_buffering(struct s2n_connection *conn, bool enabled)
+{
+    POSIX_ENSURE_REF(conn);
+    /* QUIC support is not currently compatible with recv_buffering */
+    POSIX_ENSURE(!s2n_connection_is_quic_enabled(conn), S2N_ERR_INVALID_STATE);
+    conn->recv_buffering = enabled;
+    return S2N_SUCCESS;
+}
